/**
 * Copyright 2008 Bluestem Software LLC.  All Rights Reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

package org.bluestemsoftware.open.eoa.ext.dependency.maven20;

import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import org.apache.commons.io.IOUtils;
import org.apache.maven.artifact.Artifact;
import org.apache.maven.artifact.factory.ArtifactFactory;
import org.apache.maven.artifact.manager.WagonManager;
import org.apache.maven.artifact.metadata.ArtifactMetadataSource;
import org.apache.maven.artifact.repository.ArtifactRepository;
import org.apache.maven.artifact.resolver.ArtifactCollector;
import org.apache.maven.artifact.resolver.ArtifactNotFoundException;
import org.apache.maven.artifact.resolver.ArtifactResolutionResult;
import org.apache.maven.artifact.resolver.ArtifactResolver;
import org.apache.maven.artifact.resolver.DebugResolutionListener;
import org.apache.maven.artifact.resolver.ResolutionListener;
import org.apache.maven.artifact.resolver.ResolutionNode;
import org.apache.maven.artifact.resolver.WarningResolutionListener;
import org.apache.maven.artifact.resolver.filter.AndArtifactFilter;
import org.apache.maven.artifact.resolver.filter.ArtifactFilter;
import org.apache.maven.artifact.resolver.filter.ExcludesArtifactFilter;
import org.apache.maven.artifact.resolver.filter.ScopeArtifactFilter;
import org.apache.maven.artifact.resolver.filter.TypeArtifactFilter;
import org.apache.maven.artifact.versioning.InvalidVersionSpecificationException;
import org.apache.maven.artifact.versioning.VersionRange;
import org.apache.maven.embedder.Configuration;
import org.apache.maven.embedder.DefaultConfiguration;
import org.apache.maven.embedder.MavenEmbedder;
import org.apache.maven.embedder.MavenEmbedderException;
import org.apache.maven.embedder.PlexusLoggerAdapter;
import org.apache.maven.model.Dependency;
import org.apache.maven.model.DependencyManagement;
import org.apache.maven.model.Exclusion;
import org.apache.maven.project.InvalidProjectModelException;
import org.apache.maven.project.MavenProject;
import org.apache.maven.project.MavenProjectBuilder;
import org.apache.maven.project.ProjectBuildingException;
import org.apache.maven.project.validation.ModelValidationResult;
import org.apache.maven.settings.Settings;
import org.bluestemsoftware.open.eoa.ext.dependency.maven20.dependency.AbstractDependency;
import org.bluestemsoftware.open.eoa.ext.dependency.maven20.dependency.AbstractDeploymentDependency;
import org.bluestemsoftware.open.eoa.ext.dependency.maven20.dependency.ComponentDependencyImpl;
import org.bluestemsoftware.open.eoa.ext.dependency.maven20.dependency.FactoryDependencyImpl;
import org.bluestemsoftware.open.eoa.ext.dependency.maven20.dependency.FeatureDependencyImpl;
import org.bluestemsoftware.open.eoa.ext.dependency.maven20.dependency.OtherDependencyImpl;
import org.bluestemsoftware.open.eoa.ext.dependency.maven20.dependency.ProvidedDependencyImpl;
import org.bluestemsoftware.open.eoa.ext.dependency.maven20.dependency.ScopedDependencyImpl;
import org.bluestemsoftware.open.eoa.ext.dependency.maven20.dependency.SharedDependencyImpl;
import org.bluestemsoftware.open.eoa.ext.dependency.maven20.util.InterpolationHelper;
import org.bluestemsoftware.open.eoa.ext.dependency.maven20.util.MavenEmbedderLogger;
import org.bluestemsoftware.open.eoa.ext.dependency.maven20.util.MavenException;
import org.bluestemsoftware.specification.eoa.ComponentDependency;
import org.bluestemsoftware.specification.eoa.FactoryDependency;
import org.bluestemsoftware.specification.eoa.FeatureDependency;
import org.bluestemsoftware.specification.eoa.OtherDependency;
import org.bluestemsoftware.specification.eoa.ProvidedDependency;
import org.bluestemsoftware.specification.eoa.ScopedDependency;
import org.bluestemsoftware.specification.eoa.SharedDependency;
import org.bluestemsoftware.specification.eoa.SystemDependencies;
import org.bluestemsoftware.specification.eoa.ext.ExtensionException;
import org.bluestemsoftware.specification.eoa.ext.dependency.maven20.MavenComponentDependency;
import org.bluestemsoftware.specification.eoa.ext.dependency.maven20.MavenDependencyManager;
import org.bluestemsoftware.specification.eoa.ext.dependency.maven20.MavenFactoryDependency;
import org.bluestemsoftware.specification.eoa.ext.dependency.maven20.MavenFeatureDependency;
import org.bluestemsoftware.specification.eoa.ext.dependency.maven20.MavenOtherDependency;
import org.bluestemsoftware.specification.eoa.ext.dependency.maven20.MavenProvidedDependency;
import org.bluestemsoftware.specification.eoa.ext.dependency.maven20.MavenScopedDependency;
import org.bluestemsoftware.specification.eoa.ext.dependency.maven20.MavenSharedDependency;
import org.bluestemsoftware.specification.eoa.system.SystemContext;
import org.bluestemsoftware.specification.eoa.system.System.Log;
import org.codehaus.plexus.PlexusContainer;
import org.codehaus.plexus.component.repository.exception.ComponentLookupException;

/**
 * A maven 2.0 implementation of <code>DependencyManager</code>, i.e. has implied extension ref
 * http://maven.apache.org/POM/4.0.0.
 */
public final class DependencyManagerImpl extends MavenDependencyManager {

    private static Log log = SystemContext.getContext().getSystem().getLog(System.class);

    private MavenEmbedder maven;
    private PlexusContainer mavenContainer;
    private ArtifactFactory artifactFactory;
    private MavenProjectBuilder projectBuilder;
    private ArtifactMetadataSource metadataSource;
    private ArtifactResolver artifactResolver;
    private ArtifactCollector artifactCollector;
    private AndArtifactFilter artifactFilter;
    private OrArtifactFilter otherTypeFilter;
    private Artifact dummyArtifact;
    private MavenEmbedderLogger embedderLogger;
    private List<ResolutionListener> listeners;
    private File tempDirectory;
    private Properties systemProperties = null;
    private Properties compliantSystemProperties = null;
    private List<?> remoteRepositories; // from context of system project

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.bluestemsoftware.specification.eoa.system.DependencyManager$Provider#spi_init(java
     * .io.File)
     */
    public void init(File settings) throws MavenException {

        log.info("starting maven embedded");

        ClassLoader systemClassLoader = Thread.currentThread().getContextClassLoader();

        Configuration configuration = new DefaultConfiguration();
        configuration.setUserSettingsFile(settings);
        embedderLogger = new MavenEmbedderLogger();
        configuration.setMavenEmbedderLogger(embedderLogger);
        configuration.setClassLoader(systemClassLoader);

        // create listeners which will log dependency resolution errors to our logger

        listeners = new ArrayList<ResolutionListener>();
        listeners.add(new WarningResolutionListener(new PlexusLoggerAdapter(embedderLogger)));
        if (embedderLogger.isDebugEnabled()) {
            listeners.add(new DebugResolutionListener(new PlexusLoggerAdapter(embedderLogger)));
        }

        // as of 3.0-alpha2, maven embedder iterates over all system properties
        // and casts value to a string. well ... in theory all values SHOULD
        // be a string based upon the Properties interface, but because it extends
        // HashTable objects are allowed. so ...

        try {
            systemProperties = System.getProperties();
            compliantSystemProperties = new Properties();
            for (Map.Entry<Object, Object> e : systemProperties.entrySet()) {
                if (e.getValue() instanceof String) {
                    compliantSystemProperties.put(e.getKey(), e.getValue());
                }
            }
            System.setProperties(compliantSystemProperties);
            maven = new MavenEmbedder(configuration);
        } catch (Exception ex) {
            throw new MavenException(ex);
        } finally {
            System.setProperties(systemProperties);
            Thread.currentThread().setContextClassLoader(systemClassLoader);
        }

        mavenContainer = maven.getPlexusContainer();

        // note that as of 2.1 SNAPSHOT, maven embedder ignores all settings
        // other than local repo path and profiles. servers and proxies are
        // ignored. for now we'll at least set offline flag

        try {
            Settings mavenSettings = maven.getSettings();
            WagonManager wagonManager = (WagonManager)mavenContainer.lookup(WagonManager.ROLE);
            wagonManager.setOnline(!mavenSettings.isOffline());
        } catch (ComponentLookupException ce) {
            throw new MavenException(ce);
        }

        try {
            artifactFactory = (ArtifactFactory)mavenContainer.lookup(ArtifactFactory.ROLE);
            projectBuilder = (MavenProjectBuilder)mavenContainer.lookup(MavenProjectBuilder.class);
            metadataSource = (ArtifactMetadataSource)mavenContainer.lookup(ArtifactMetadataSource.class);
            artifactCollector = (ArtifactCollector)mavenContainer.lookup(ArtifactCollector.class);
            artifactResolver = (ArtifactResolver)mavenContainer.lookup(ArtifactResolver.class);
        } catch (ComponentLookupException ce) {
            throw new MavenException(ce);
        }
        
        artifactFilter = new AndArtifactFilter();
        ScopeArtifactFilter scopeFilter = new ScopeArtifactFilter(Artifact.SCOPE_RUNTIME);
        artifactFilter.add(scopeFilter); 
        TypeArtifactFilter typeFilter = new TypeArtifactFilter("jar");
        artifactFilter.add(typeFilter);

        otherTypeFilter = new OrArtifactFilter();
        otherTypeFilter.add(new TypeArtifactFilter("eoa-component"));
        otherTypeFilter.add(new TypeArtifactFilter("eoa-factory"));
        otherTypeFilter.add(new TypeArtifactFilter("eoa-feature"));

        dummyArtifact = artifactFactory.createArtifact("dummy", "dummy", "dummy", Artifact.SCOPE_RUNTIME, "jar");

        tempDirectory = SystemContext.getContext().getSystem().getSystemTmpDir();

        log.trace("init end");
    }

    public void destroy() {
        if (maven != null) {
            try {
                maven.stop();
            } catch (MavenEmbedderException ex) {
                log.error("error stopping maven embedded. " + ex);
            }
            maven = null;
            mavenContainer = null;
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.bluestemsoftware.specification.eoa.system.DependencyManager#modelDependencies(java
     * .io.File)
     */
    public SystemDependencies modelSystemDependencies(File projectDescriptor) throws MavenException {

        log.info("modeling system dependencies");

        if (projectDescriptor == null) {
            throw new IllegalArgumentException("projectDescriptor null");
        }

        try {
            
            System.setProperties(compliantSystemProperties);

            MavenProject project = null;
            try {
                File temp = replaceSystemPathVariables(projectDescriptor);
                project = maven.readProject(temp);
                temp.deleteOnExit();
                remoteRepositories = project.getRemoteArtifactRepositories();
                project.setDependencyArtifacts(getProjectDependencies(null, project));
            } catch (Exception ex) {
                if (ex instanceof InvalidProjectModelException) {
                    throw new MavenException("Error modeling system dependencies. Invalid system POM. "
                            + ((InvalidProjectModelException)ex).getValidationResult().toString());
                } else {
                    throw new MavenException("Error modeling system dependencies. " + ex);
                }
            }

            // retrieve model identifiers
            String organizationID = project.getGroupId();
            String artifactID = project.getArtifactId();
            String version = project.getVersion();

            // retrieve dependencies declared explicitly within pom, i.e.
            // no transitive dependencies

            Set<?> explicitArtifacts = project.getDependencyArtifacts();

            // iterate over explicit artifacts and add any jar dependencies with
            // provided scope to dependency's list of shared dependencies. do not
            // process transitively

            log.info("resolving shared dependencies ...");

            List<SharedDependency> sharedDependencies = new ArrayList<SharedDependency>();

            Iterator<?> itr = explicitArtifacts.iterator();
            while (itr.hasNext()) {

                Artifact artifact = (Artifact)itr.next();

                if (!artifact.getScope().equals(MavenSharedDependency.SCOPE)) {
                    continue;
                }
                if (artifact.getType().equals(MavenSharedDependency.TYPE)) {
                    SharedDependencyImpl sharedDependency = new SharedDependencyImpl(artifact.getFile(), artifact
                            .getGroupId(), artifact.getArtifactId(), artifact.getVersion(), artifact.getType(),
                            !artifact.isOptional());
                    sharedDependencies.add(sharedDependency);
                    log.trace("created shared dependency " + artifact.getArtifactId());
                }

            }

            // iterate over extension factory dependencies explicitly declared within
            // pom and model each. managed dependencies can specify extension
            // dependencies, i.e. type is 'eoa-factory' and can define feature
            // dependencies, i.e. type is 'eoa-feature'. transitive dependencies
            // are modeled within each returned object. don't worry about circular
            // refs because maven checks this for us

            log.info("resolving factory dependencies ...");

            List<FactoryDependency> extensionDependencies = new ArrayList<FactoryDependency>();

            itr = explicitArtifacts.iterator();
            while (itr.hasNext()) {
                Artifact artifact = (Artifact)itr.next();
                if (!artifact.getScope().equals(MavenFactoryDependency.SCOPE)) {
                    if (!artifact.getScope().equals(MavenFeatureDependency.SCOPE)) {
                        continue;
                    }
                }
                if (!artifact.getType().equals(MavenFactoryDependency.TYPE)) {
                    if (!artifact.getType().equals(MavenFeatureDependency.TYPE)) {
                        continue;
                    }
                }
                FactoryDependency efd = modelFactoryDependency(artifact, project, new HashSet<String>());
                if (extensionDependencies.contains(efd)) {
                    continue;
                } else {
                    extensionDependencies.add(efd);
                }
            }

            // model component dependencies - type is eoa-component. component dependencies
            // can define component dependencies, other dependencies and feature dependencies

            List<ComponentDependency> componentDependencies = new ArrayList<ComponentDependency>();

            itr = explicitArtifacts.iterator();
            while (itr.hasNext()) {

                Artifact artifact = (Artifact)itr.next();
                if (!artifact.getScope().equals(MavenComponentDependency.SCOPE)) {
                    continue;
                }
                if (!artifact.getType().equals(MavenComponentDependency.TYPE)) {
                    continue;
                }

                ComponentDependency cd = modelComponentDependency(artifact, project, new HashSet<String>(), false);
                if (cd == null || componentDependencies.contains(cd)) {
                    continue;
                } else {
                    componentDependencies.add(cd);
                }

            }

            log.info("modeled system dependencies");
            return new SystemDependencies(organizationID, artifactID, version, sharedDependencies,
                    extensionDependencies, componentDependencies);

        } finally {
            System.setProperties(systemProperties);
        }

    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.bluestemsoftware.specification.eoa.system.DependencyManager#resolveDependency(java
     * .lang.String, java.lang.String, java.lang.String, java.lang.String)
     */
    @Override
    public org.bluestemsoftware.specification.eoa.Dependency resolveDependency(String organizationID, String artifactID, String version, String extension) throws ExtensionException {
        Artifact artifact = artifactFactory.createBuildArtifact(organizationID, artifactID, version, extension);
        try {
            resolveArtifact(artifact, remoteRepositories);
        } catch (Exception ex) {
            throw new MavenException("Error resolving dependency. " + ex.getMessage());
        }
        return new AbstractDependency(artifact.getFile(), organizationID, artifactID, version, extension);
    }

    /** ***************************** restricted methods ************************* */

    @SuppressWarnings("unchecked")
    private FactoryDependency modelFactoryDependency(Artifact extensionFactoryArtifact, MavenProject rootProject, Set<String> processed) throws MavenException {

        String ref = extensionFactoryArtifact.getGroupId()
                + "/"
                + extensionFactoryArtifact.getArtifactId()
                + "/"
                + extensionFactoryArtifact.getVersion()
                + "/"
                + extensionFactoryArtifact.getType();

        if (log.isTraceEnabled()) {
            log.trace("resolving " + extensionFactoryArtifact.getArtifactId() + " ...");
        }

        // model factory project and retrieve dependencies declared explicitly within pom, i.e.
        // no transitive dependencies

        MavenProject extensionFactoryProject;
        try {
            extensionFactoryProject = modelDependencyProject(extensionFactoryArtifact, rootProject);
        } catch (Exception ex) {
            throw new MavenException("Error modeling factory project "
                    + extensionFactoryArtifact.getArtifactId()
                    + ". "
                    + ex);
        }
        Set<Artifact> explicitArtifacts = extensionFactoryProject.getDependencyArtifacts();

        // iterate over explicit artifacts and add any jar dependencies with
        // system scope to dependency's list of provided dependencies. do not
        // process transitively

        List<ProvidedDependency> providedDependencies = new ArrayList<ProvidedDependency>();

        Iterator<?> itr = explicitArtifacts.iterator();
        while (itr.hasNext()) {

            Artifact artifact = (Artifact)itr.next();

            if (!artifact.getScope().equals(MavenProvidedDependency.SCOPE)) {
                continue;
            }
            if (artifact.getType().equals(MavenProvidedDependency.TYPE)) {
                ProvidedDependencyImpl providedDependency = new ProvidedDependencyImpl(artifact.getFile(),
                        artifact.getGroupId(), artifact.getArtifactId(), artifact.getVersion(), artifact.getType());
                providedDependencies.add(providedDependency);
                if (log.isDebugEnabled()) {
                    log.debug("created provided dependency " + providedDependency.getRef());
                }
            }

        }

        // iterate over explicit artifacts and add any jar dependencies with provided
        // scope not marked as optional to dependency's list of shared dependencies.
        // do not process transitively

        List<SharedDependency> sharedDependencies = new ArrayList<SharedDependency>();

        itr = explicitArtifacts.iterator();
        while (itr.hasNext()) {

            Artifact artifact = (Artifact)itr.next();

            if (!artifact.getScope().equals(MavenSharedDependency.SCOPE)) {
                continue;
            }
            if (artifact.getType().equals(MavenSharedDependency.TYPE)) {
                if (artifact.isOptional()) {
                    continue;
                }
                SharedDependencyImpl sharedDependency = new SharedDependencyImpl(artifact.getFile(), artifact
                        .getGroupId(), artifact.getArtifactId(), artifact.getVersion(), artifact.getType(),
                        !artifact.isOptional());
                sharedDependencies.add(sharedDependency);
                if (log.isDebugEnabled()) {
                    log.debug("created shared dependency " + sharedDependency.getRef());
                }
            }

        }

        // iterate over explicit artifacts and add any jar dependencies with
        // scope 'compile' and optional equals true to list of scoped artifacts

        Set<Artifact> scopedArtifacts = new HashSet<Artifact>();

        itr = explicitArtifacts.iterator();
        while (itr.hasNext()) {
            Artifact artifact = (Artifact)itr.next();
            if (!artifact.getScope().equals(MavenScopedDependency.SCOPE)) {
                continue;
            }
            if (artifact.getType().equals(MavenScopedDependency.TYPE)) {
                if (!artifact.isOptional()) {
                    continue;
                }
                scopedArtifacts.add(artifact);
            }
        }
        
        // before we collect transitive artifacts, validate poms for each
        // scoped artifacts. as of version 3.0-alpha2, problem POMS are
        // just logged w/ debug level logging and the artifact's deps
        // are ignored
        
        for (Artifact a : scopedArtifacts) {
            try {
                projectBuilder.buildFromRepository( a, remoteRepositories, maven.getLocalRepository() );
            } catch ( InvalidProjectModelException e ) {
                ModelValidationResult validationResult = e.getValidationResult();
                StringBuilder sb = new StringBuilder();
                String r = a.getGroupId()
                + "/"
                + a.getArtifactId()
                + "/"
                + a.getVersion()
                + "/"
                + a.getType();
                sb.append("POM for dependency ");
                sb.append(r);
                sb.append(" failed to validate with the following errors: ");
                if ( validationResult != null ) {
                    for ( Iterator i = validationResult.getMessages().iterator(); i.hasNext(); ) {
                        sb.append(i.next().toString());
                    }
                } else {
                    sb.append("unknown");
                }
                throw new MavenException("Error modeling factory project "
                        + extensionFactoryArtifact.getArtifactId()
                        + ". "
                        + sb.toString());
            } catch (ProjectBuildingException pe) {
                throw new MavenException("Error modeling factory project "
                        + extensionFactoryArtifact.getArtifactId()
                        + ". "
                        + pe.getMessage());
            }
        }

        // collect transitive dependencies defined by scoped artifacts, i.e.
        // artifacts with scope compile or runtime and not marked as optional

        ArtifactResolutionResult arr;
        try {
            Map<String, Artifact> managedVersionMap = createManagedVersionMap(extensionFactoryProject);
            arr = artifactCollector.collect(scopedArtifacts, dummyArtifact, managedVersionMap, maven
                    .getLocalRepository(), extensionFactoryProject.getRemoteArtifactRepositories(),
                    metadataSource, artifactFilter, listeners);
        } catch (Exception ex) {
            throw new MavenException("Error resolving transitive dependencies. " + ex);
        }

        // iterate over each dependency node and resolve the artifact which may
        // involve retrieving from a remote repo, i.e. log any errors

        Set<Artifact> resolvedArtifacts = new HashSet<Artifact>();
        itr = arr.getArtifactResolutionNodes().iterator();
        while (itr.hasNext()) {
            ResolutionNode node = (ResolutionNode)itr.next();
            try {
                Artifact ra = node.getArtifact();
                resolveArtifact(ra, node.getRemoteRepositories());
                resolvedArtifacts.add(ra);
            } catch (Exception ex) {
                log.warn("error resolving " + node.getArtifact().toString() + ". " + ex.getMessage());
            }
        }

        // now iterate over scoped artifacts and all transitive artifacts
        // and create a scoped dependency object for each

        List<ScopedDependency> scopedDependencies = new ArrayList<ScopedDependency>();

        for (Artifact ra : resolvedArtifacts) {
            ScopedDependencyImpl scopedDependency = new ScopedDependencyImpl(ra.getFile(), ra.getGroupId(), ra
                    .getArtifactId(), ra.getVersion(), ra.getType(), ra.getDependencyTrail().toString());
            scopedDependencies.add(scopedDependency);
            log.debug("created scoped dependency " + scopedDependency.getRef());
            log.debug("dependency trail: " + ra.getDependencyTrail());
        }

        // as an optimization, we expand deployment artifact in place,
        // i.e. within same parent dir as compressed version

        boolean isSnapshot = extensionFactoryArtifact.getVersion().contains("SNAPSHOT");
        File file = expandDeploymentArtifact(extensionFactoryArtifact.getFile(), isSnapshot);

        // iterate over feature factory dependencies, i.e. any artifact with type
        // eoa-feature and scope runtime and process them recursively. extension
        // factories are only allowed to define feature dependencies, i.e.
        // dependencies on non-feature extension factory types are not allowed.
        // if we've already processed this dependency somewhere up the hierarchy
        // do not recurse again. omit the feature dependencies and system will
        // error out when system deps are processed

        List<FeatureDependency> featureDependencies = new ArrayList<FeatureDependency>();

        itr = explicitArtifacts.iterator();
        while (itr.hasNext()) {
            Artifact artifact = (Artifact)itr.next();
            if (!artifact.getScope().equals(MavenFeatureDependency.SCOPE)) {
                continue;
            }
            if (!artifact.getType().equals(MavenFeatureDependency.TYPE)) {
                continue;
            }
            String groupID = artifact.getGroupId();
            String artifactID = artifact.getArtifactId();
            String version = artifact.getVersion();
            String type = artifact.getType();
            String vref = groupID + "/" + artifactID + "/" + type;
            FeatureDependency ffd = null;
            if (processed.contains(vref)) {
                boolean isRequired = !artifact.isOptional();
                ffd = new FeatureDependencyImpl(file, groupID, artifactID, version, type, isRequired);
            } else {
                processed.add(vref);
                ffd = (FeatureDependency)modelFactoryDependency(artifact, rootProject, processed);
            }
            if (featureDependencies.contains(ffd)) {
                continue;
            } else {
                featureDependencies.add(ffd);
            }
            log.debug("created feature dependency " + ffd.getRef());
        }

        // create the factory dependency. note that we must test to see if this is
        // an extension factory dependency other than a feature factory dependency,
        // i.e. which was listed within root of system dependency model

        FactoryDependency fd = null;

        if (extensionFactoryArtifact.getType().equals(MavenFeatureDependency.TYPE)) {
            boolean isRequired = !extensionFactoryArtifact.isOptional();
            fd = new FeatureDependencyImpl(file, extensionFactoryArtifact.getGroupId(), extensionFactoryArtifact
                    .getArtifactId(), extensionFactoryArtifact.getVersion(), extensionFactoryArtifact.getType(),
                    isRequired);

        } else {
            fd = new FactoryDependencyImpl(file, extensionFactoryArtifact.getGroupId(), extensionFactoryArtifact
                    .getArtifactId(), extensionFactoryArtifact.getVersion(), extensionFactoryArtifact.getType());
        }

        // iterate over explicit artifacts and add any dependency other than a
        // dependency with type 'jar', or 'eoa-feature' to dependency's list of
        // other dependencies. do not process transitively

        Map<String, OtherDependency> otherDependencies = new HashMap<String, OtherDependency>();

        itr = explicitArtifacts.iterator();
        while (itr.hasNext()) {
            Artifact artifact = (Artifact)itr.next();
            if (!otherTypeFilter.include(artifact) && artifact.getScope().equals(MavenOtherDependency.SCOPE)) {
                OtherDependencyImpl otherDependency = new OtherDependencyImpl(artifact.getFile(), artifact
                        .getGroupId(), artifact.getArtifactId(), artifact.getVersion(), artifact.getType());
                if (otherDependencies.containsKey(otherDependency.getVersionlessRef())) {
                    throw new MavenException("More than one version of 'other' dependency "
                            + otherDependency.getVersionlessRef()
                            + " specified by deployment "
                            + fd.getRef()
                            + ".");
                }
                otherDependencies.put(otherDependency.getVersionlessRef(), otherDependency);
                if (log.isDebugEnabled()) {
                    log.debug("created other dependency " + otherDependency.getRef());
                }
            }
        }

        ((AbstractDeploymentDependency)fd).setProvidedDependencies(providedDependencies);
        ((AbstractDeploymentDependency)fd).setSharedDependencies(sharedDependencies);
        ((AbstractDeploymentDependency)fd).setScopedDependencies(scopedDependencies);
        ((AbstractDeploymentDependency)fd).setFeatureDependencies(featureDependencies);
        ((AbstractDeploymentDependency)fd).setOtherDependencies(otherDependencies);

        processed.add(ref);

        return fd;

    }

    private MavenProject modelDependencyProject(Artifact factoryArtifact, MavenProject parentProject) throws Exception {

        String ref = factoryArtifact.getGroupId()
                + "/"
                + factoryArtifact.getArtifactId()
                + "/"
                + factoryArtifact.getVersion()
                + "/"
                + factoryArtifact.getType();

        // check to see if we're running within context of factory test harness. if
        // so, and dependency artifact models 'deployment under test', retrieve pom
        // from 'basedir' rather than building project from repository

        String dutRef = null;
        if (systemProperties.get("eoa.system.test.properties") != null) {
            Map<?, ?> testProperties = (Map<?, ?>)systemProperties.get("eoa.system.test.properties");
            dutRef = testProperties.get("organizationID")
                    + "/"
                    + testProperties.get("artifactID")
                    + "/"
                    + testProperties.get("version")
                    + "/"
                    + testProperties.get("extension");
        }

        MavenProject dependencyProject = null;
        if (dutRef != null && dutRef.equals(ref)) {
            String basedir = System.getProperty("basedir");
            File factoryUnderTestPOM = new File(basedir, "pom.xml");
            factoryUnderTestPOM = replaceSystemPathVariables(factoryUnderTestPOM);
            try {
                dependencyProject = maven.readProject(factoryUnderTestPOM);
            } catch (org.apache.maven.project.InvalidProjectModelException ie) {
                throw new MavenException("Error validating POM at location '"
                        + new File(basedir, "pom.xml")
                        + "'. "
                        + ie.getValidationResult());
            }
            factoryUnderTestPOM.deleteOnExit();
            dependencyProject.setDependencyArtifacts(getProjectDependencies(factoryArtifact, dependencyProject));
        } else {
            Artifact pomArtifact = artifactFactory.createProjectArtifact(factoryArtifact.getGroupId(),
                    factoryArtifact.getArtifactId(), factoryArtifact.getVersion());
            resolveArtifact(pomArtifact, parentProject.getRemoteArtifactRepositories());
            File temp = pomArtifact.getFile();
            pomArtifact.setFile(replaceSystemPathVariables(temp));
            try {
                dependencyProject = projectBuilder.buildFromRepository(pomArtifact, parentProject
                        .getRemoteArtifactRepositories(), maven.getLocalRepository());
            } catch (org.apache.maven.project.InvalidProjectModelException ie) {
                throw new MavenException("Error validating POM at location '"
                        + temp.getAbsolutePath()
                        + "'. "
                        + ie.getValidationResult());
            }
            pomArtifact.getFile().deleteOnExit();
            dependencyProject.setDependencyArtifacts(getProjectDependencies(factoryArtifact, dependencyProject));
        }

        return dependencyProject;

    }

    /*
     * this is a workaround for what i think is a bug in maven project builder, i.e. env vars
     * in systempath value (scope=system) are not resolved before validating project model (as
     * of version 2.1 snapshot).
     */
    private File replaceSystemPathVariables(File file) throws Exception {
        String input = IOUtils.toString(new FileInputStream(file), "UTF-8");
        input = InterpolationHelper.interpolate(input);
        File tempDir = SystemContext.getContext().getSystem().getSystemTmpDir();
        File temp = File.createTempFile("tmp", ".pom", tempDir);
        FileOutputStream out = new FileOutputStream(temp);
        ByteArrayInputStream in = new ByteArrayInputStream(input.getBytes("UTF-8"));
        IOUtils.copy(in, out);
        out.flush();
        out.close();
        return temp;
    }

    @SuppressWarnings("unchecked")
    private ComponentDependency modelComponentDependency(Artifact componentArtifact, MavenProject rootProject, Set<String> processed, boolean isTransitive) throws MavenException {

        // if we've already processed the component dependency somewhere
        // up the hierarchy, we quietly ignore it. circular refs are
        // allowed, i.e. two application deployments may depend on
        // one another at runtime, i.e. they may invoke one another. note
        // that although we are creating a hierarchical model here, it
        // effectively ends up being flattened into a set of dependencies
        // for which the associated component deployment must be deployed

        String ref = componentArtifact.getGroupId()
                + "/"
                + componentArtifact.getArtifactId()
                + "/"
                + componentArtifact.getVersion()
                + "/"
                + componentArtifact.getType();

        if (processed.contains(ref)) {
            return null;
        }

        log.debug("resolving " + componentArtifact.getArtifactId() + " ...");

        // model component project and retrieve dependencies declared explicitly within
        // pom, i.e. no transitive dependencies

        MavenProject componentProject;
        try {
            componentProject = modelDependencyProject(componentArtifact, rootProject);
        } catch (Exception ex) {
            throw new MavenException("Error modeling component project " + ref + ". " + ex);
        }
        Set<Artifact> explicitArtifacts = componentProject.getDependencyArtifacts();

        // iterate over explicit artifacts and add any jar dependencies with
        // system scope to dependency's list of provided dependencies. do not
        // process transitively. note that if we're processing a transitive
        // component dependency we do not process classpath artifacts

        List<ProvidedDependency> providedDependencies = new ArrayList<ProvidedDependency>();

        if (!isTransitive) {
            Iterator<?> itr = explicitArtifacts.iterator();
            while (itr.hasNext()) {

                Artifact artifact = (Artifact)itr.next();

                if (!artifact.getScope().equals(MavenProvidedDependency.SCOPE)) {
                    continue;
                }
                if (artifact.getType().equals(MavenProvidedDependency.TYPE)) {
                    ProvidedDependencyImpl providedDependency = new ProvidedDependencyImpl(artifact.getFile(),
                            artifact.getGroupId(), artifact.getArtifactId(), artifact.getVersion(), artifact
                                    .getType());
                    providedDependencies.add(providedDependency);
                    if (log.isDebugEnabled()) {
                        log.debug("created provided dependency " + providedDependency.getRef());
                    }
                }

            }
        }

        // iterate over explicit artifacts and add any jar dependencies with provided
        // scope not marked as optional to dependency's list of shared dependencies.
        // do not process transitively. note that if we're processing a transitive
        // component dependency we do not process classpath artifacts

        List<SharedDependency> sharedDependencies = new ArrayList<SharedDependency>();

        if (!isTransitive) {
            Iterator<?> itr = explicitArtifacts.iterator();
            while (itr.hasNext()) {

                Artifact artifact = (Artifact)itr.next();

                if (!artifact.getScope().equals(MavenSharedDependency.SCOPE)) {
                    continue;
                }
                if (artifact.getType().equals(MavenSharedDependency.TYPE)) {
                    if (artifact.isOptional()) {
                        continue;
                    }
                    SharedDependencyImpl sharedDependency = new SharedDependencyImpl(artifact.getFile(), artifact
                            .getGroupId(), artifact.getArtifactId(), artifact.getVersion(), artifact.getType(),
                            !artifact.isOptional());
                    sharedDependencies.add(sharedDependency);
                    if (log.isDebugEnabled()) {
                        log.debug("created shared dependency " + sharedDependency.getRef());
                    }
                }

            }
        }

        // iterate over explicit artifacts and add any jar dependencies with
        // scope 'compile' and optional equals true to list of scoped artifacts
        // note that if we're processing a transitive component dependency we
        // do not process classpath artifacts

        List<ScopedDependency> scopedDependencies = new ArrayList<ScopedDependency>();

        if (!isTransitive) {

            Set<Artifact> scopedArtifacts = new HashSet<Artifact>();

            Iterator<?> itr = explicitArtifacts.iterator();
            while (itr.hasNext()) {
                Artifact artifact = (Artifact)itr.next();
                if (!artifact.getScope().equals(MavenScopedDependency.SCOPE)) {
                    continue;
                }
                if (artifact.getType().equals(MavenScopedDependency.TYPE)) {
                    if (!artifact.isOptional()) {
                        continue;
                    }
                    scopedArtifacts.add(artifact);
                }
            }

            // collect transitive dependencies defined by scoped artifacts, i.e.
            // artifacts with scope compile or runtime and not marked as optional

            ArtifactResolutionResult arr;
            try {
                Map<String, Artifact> managedVersionMap = createManagedVersionMap(componentProject);
                arr = artifactCollector.collect(scopedArtifacts, dummyArtifact, managedVersionMap, maven
                        .getLocalRepository(), componentProject.getRemoteArtifactRepositories(), metadataSource,
                        artifactFilter, listeners);
            } catch (Exception ex) {
                throw new MavenException("Error resolving transitive dependencies. " + ex);
            }

            // iterate over each dependency node and resolve the artifact which may
            // involve retrieving from a remote repo, i.e. log any errors

            Set<Artifact> resolvedArtifacts = new HashSet<Artifact>();
            itr = arr.getArtifactResolutionNodes().iterator();
            while (itr.hasNext()) {
                ResolutionNode node = (ResolutionNode)itr.next();
                try {
                    Artifact ra = node.getArtifact();
                    resolveArtifact(ra, node.getRemoteRepositories());
                    resolvedArtifacts.add(ra);
                } catch (Exception ex) {
                    log.warn("error resolving " + node.getArtifact().toString() + ". " + ex.getMessage());
                }
            }

            // now iterate over scoped artifacts and all transitive artifacts
            // and create a scoped dependency object for each

            for (Artifact ra : resolvedArtifacts) {
                ScopedDependencyImpl scopedDependency = new ScopedDependencyImpl(ra.getFile(), ra.getGroupId(), ra
                        .getArtifactId(), ra.getVersion(), ra.getType(), ra.getDependencyTrail().toString());
                scopedDependencies.add(scopedDependency);
                log.debug("created scoped dependency " + scopedDependency.getRef());
                log.debug("dependency trail: " + ra.getDependencyTrail());
            }

        }

        // as an optimization, we expand deployment artifact in place,
        // i.e. within same parent dir as compressed version

        boolean isSnapshot = componentArtifact.getVersion().contains("SNAPSHOT");
        File file = expandDeploymentArtifact(componentArtifact.getFile(), isSnapshot);

        ComponentDependency cd = new ComponentDependencyImpl(file, componentArtifact.getGroupId(),
                componentArtifact.getArtifactId(), componentArtifact.getVersion(), componentArtifact.getType());

        // iterate over component dependencies, i.e. any artifact with type
        // eoa-component and scope runtime. note that if this dependency is
        // marked as optional, we do not process transitive component
        // dependencies

        List<ComponentDependency> componentDependencies = new ArrayList<ComponentDependency>();

        if (!componentArtifact.isOptional()) {

            Iterator<?> itr = explicitArtifacts.iterator();
            while (itr.hasNext()) {

                Artifact artifact = (Artifact)itr.next();
                if (!artifact.getScope().equals(MavenComponentDependency.SCOPE)) {
                    continue;
                }
                if (!artifact.getType().equals(MavenComponentDependency.TYPE)) {
                    continue;
                }

                ComponentDependency temp = modelComponentDependency(artifact, rootProject, processed, true);

                if (temp == null || componentDependencies.contains(temp)) {
                    continue;
                } else {
                    componentDependencies.add(temp);
                }

            }

        }

        // iterate over feature dependencies, i.e. any artifact with type
        // eoa-feature and scope runtime. note that if we're processing a transitive
        // component dependency we do not process feature dependencies, i.e.
        // runtime engine defs are only loaded if defined within a root
        // component dependency. no engine - no required features (features
        // required by runtime bindings are defined within binding factory
        // deployment which must be declared within root of system pom)

        List<FeatureDependency> featureDependencies = new ArrayList<FeatureDependency>();

        if (!isTransitive) {
            Iterator<?> itr = explicitArtifacts.iterator();
            while (itr.hasNext()) {
                Artifact artifact = (Artifact)itr.next();
                if (!artifact.getScope().equals(MavenFeatureDependency.SCOPE)) {
                    continue;
                }
                if (!artifact.getType().equals(MavenFeatureDependency.TYPE)) {
                    continue;
                }
                FeatureDependency ffd = (FeatureDependency)modelFactoryDependency(artifact, rootProject,
                        new HashSet<String>());
                if (featureDependencies.contains(ffd)) {
                    continue;
                } else {
                    featureDependencies.add(ffd);
                }
                log.debug("created feature dependency " + ffd.getRef());
            }
        }

        // iterate over explicit artifacts and add any dependency other than a
        // dependency with type 'jar', 'eoa-component', or 'eoa-feature' to
        // dependency's list of other dependencies. do not process transitively.
        // note that if we're processing a transitive component dependency we do
        // not process other dependencies, i.e. runtime engine defs are only
        // loaded if defined within a root component dependency. no engine - no
        // modules to deploy

        Map<String, OtherDependency> otherDependencies = new HashMap<String, OtherDependency>();

        if (!isTransitive) {
            Iterator<?> itr = explicitArtifacts.iterator();
            while (itr.hasNext()) {
                Artifact artifact = (Artifact)itr.next();
                if (!otherTypeFilter.include(artifact) && artifact.getScope().equals(MavenOtherDependency.SCOPE)) {
                    OtherDependencyImpl otherDependency = new OtherDependencyImpl(artifact.getFile(), artifact
                            .getGroupId(), artifact.getArtifactId(), artifact.getVersion(), artifact.getType());
                    if (otherDependencies.containsKey(otherDependency.getVersionlessRef())) {
                        throw new MavenException("More than one version of 'other' dependency "
                                + otherDependency.getVersionlessRef()
                                + " specified by deployment "
                                + cd.getRef()
                                + ".");
                    }
                    otherDependencies.put(otherDependency.getVersionlessRef(), otherDependency);
                    if (log.isDebugEnabled()) {
                        log.debug("created other dependency " + otherDependency.getRef());
                    }
                }
            }
        }

        ((AbstractDeploymentDependency)cd).setProvidedDependencies(providedDependencies);
        ((AbstractDeploymentDependency)cd).setSharedDependencies(sharedDependencies);
        ((AbstractDeploymentDependency)cd).setScopedDependencies(scopedDependencies);
        ((ComponentDependencyImpl)cd).setComponentDependencies(componentDependencies);
        ((AbstractDeploymentDependency)cd).setFeatureDependencies(featureDependencies);
        ((ComponentDependencyImpl)cd).setOtherDependencies(otherDependencies);

        // add the processed componenent dependency to set of
        // processed component dependencies to prevent endless
        // loop

        processed.add(ref);

        return cd;

    }

    private Set<Artifact> getProjectDependencies(Artifact dependencyArtifact, MavenProject dependencyProject) throws Exception {

        Set<Artifact> projectDependencies = new HashSet<Artifact>();

        for (Object dependencyObj : dependencyProject.getDependencies()) {

            Dependency dependency = (Dependency)dependencyObj;
            String groupID = dependency.getGroupId();
            String artifactID = dependency.getArtifactId();
            VersionRange versionRange = VersionRange.createFromVersionSpec(dependency.getVersion());
            String scope = dependency.getScope() == null ? Artifact.SCOPE_COMPILE : dependency.getScope();
            String type = dependency.getType() == null ? "jar" : dependency.getType();
            String classifier = dependency.getClassifier();
            boolean isOptional = dependency.isOptional();

            if (scope.equals("test")) {
                continue;
            }

            Artifact artifact = artifactFactory.createDependencyArtifact(groupID, artifactID, versionRange, type,
                    classifier, scope, isOptional);

            if (scope.equals(Artifact.SCOPE_SYSTEM)) {
                File file = new File(dependency.getSystemPath());
                artifact.setFile(file);
            }

            if (dependency.getExclusions() != null) {
                List<String> patterns = new ArrayList<String>();
                for (Object exclusionObj : dependency.getExclusions()) {
                    Exclusion exclusion = (Exclusion)exclusionObj;
                    StringBuilder sb = new StringBuilder();
                    sb.append(exclusion.getGroupId());
                    sb.append(":");
                    sb.append(exclusion.getArtifactId());
                    patterns.add(sb.toString());
                }
                ExcludesArtifactFilter excludesFilter = new ExcludesArtifactFilter(patterns);
                artifact.setDependencyFilter(excludesFilter);
            }

            // check to see if we're running within context of test harness. if
            // so, and dependency is deployment under test, i.e. defined within system
            // dependencies model, set file as packaged archive within target dir
            // rather than resolving from local repo. note that phase is integration-test
            // which implies that package phase has completed

            Map<?, ?> testProperties = (Map<?, ?>)systemProperties.get("eoa.system.test.properties");
            if (testProperties != null) {
                String ref = artifact.getGroupId()
                        + "/"
                        + artifact.getArtifactId()
                        + "/"
                        + artifact.getVersion()
                        + "/"
                        + artifact.getType();
                String dutRef = testProperties.get("organizationID")
                        + "/"
                        + testProperties.get("artifactID")
                        + "/"
                        + testProperties.get("version")
                        + "/"
                        + testProperties.get("extension");
                if (ref.equals(dutRef)) {
                    String basedir = System.getProperty("basedir");
                    String archiveName = artifact.getArtifactId()
                            + "-"
                            + artifact.getVersion()
                            + "."
                            + artifact.getType();
                    artifact.setFile(new File(new File(basedir, "target/"), archiveName));
                    projectDependencies.add(artifact);
                    continue;
                }
            }

            // if we are resolving an 'other' dependency. the spec requires
            // that the artifact file be located within deployment

            if (dependencyArtifact != null) {
                if (!otherTypeFilter.include(artifact) && artifact.getScope().equals(MavenOtherDependency.SCOPE)) {
                    String path = artifact.getGroupId()
                            + "/"
                            + artifact.getArtifactId()
                            + "."
                            + artifact.getType();
                    File file = dependencyArtifact.getFile();
                    if (file.isDirectory()) {
                        File source = new File(file, path);
                        if (source.exists() == false) {
                            throw new MavenException("Error resolving 'other' dependency."
                                    + " Dependency must be located within parent project file "
                                    + file.getName()
                                    + " at path "
                                    + path);
                        }
                        artifact.setFile(source);
                    } else {
                        ZipFile zipFile = new ZipFile(file);
                        ZipEntry entry = zipFile.getEntry(path);
                        if (entry == null) {
                            throw new MavenException("Error resolving 'other' dependency."
                                    + " Dependency must be located within parent project file "
                                    + file.getName()
                                    + " at path "
                                    + path);
                        }
                        File target = File.createTempFile("eoa-", "." + artifact.getType(), tempDirectory);
                        target.deleteOnExit();
                        InputStream in = null;
                        OutputStream out = null;
                        try {
                            in = zipFile.getInputStream(entry);
                            out = new BufferedOutputStream(new FileOutputStream(target));
                            byte[] buffer = new byte[4096];
                            for (int length = 0; length >= 0; length = in.read(buffer)) {
                                out.write(buffer, 0, length);
                            }
                        } finally {
                            if (in != null) {
                                in.close();
                            }
                            if (out != null) {
                                out.close();
                            }
                        }
                        artifact.setFile(target);
                    }
                    projectDependencies.add(artifact);
                    continue;
                }
            }

            try {

                resolveArtifact(artifact, dependencyProject.getRemoteArtifactRepositories());
                projectDependencies.add(artifact);

            } catch (ArtifactNotFoundException ae) {

                // if we're running within context of a test, and we're resolving a
                // component dependency, and the dependency is not found, we will
                // log a warning and assume that the component dependency has been
                // mocked

                if (systemProperties.get("eoa.system.test.properties") != null) {
                    if (artifact.getType().equals(MavenComponentDependency.TYPE)) {
                        log.warn("Failed to resolve component dependency "
                                + ae.getGroupId()
                                + "/"
                                + ae.getArtifactId()
                                + ":"
                                + ae.getVersion()
                                + " from any repository.");
                    } else {
                        throw ae;
                    }
                } else {
                    throw ae;
                }

            }
        }

        return projectDependencies;

    }

    @SuppressWarnings("all")
    private void resolveArtifact(Artifact artifact, List<?> remoteRepositories) throws Exception {

        // if artifact is a snapshot, save off the artifact version before resolving. for
        // some reason, as of version 2.1-SNAPSHOT, the DefaultArtifactResolver overwrites
        // SNAPSHOT with timestamp version format (baseVersion and version), which will
        // create a VERY obscure bug, i.e. pom will not be located and will instead be
        // stubbed, i.e. no dependencies ...

        if (artifact.isSnapshot()) {
            String version = artifact.getVersion();
            artifactResolver.resolve(artifact, (List<ArtifactRepository>)remoteRepositories, maven
                    .getLocalRepository());
            artifact.selectVersion(version);
        } else {
            artifactResolver.resolve(artifact, (List<ArtifactRepository>)remoteRepositories, maven
                    .getLocalRepository());
        }

    }

    /*
     * as an optimization we create an uncompressed version of artifact within same parent
     * directory as compressed version. this allows system to use file as-is without
     * unarchiving to a temp file on every start and then having to clean-up on exit (slow) the
     * thought being, if version is not snapshot, image will not change
     */
    private File expandDeploymentArtifact(File file, boolean isSnapshot) throws MavenException {

        File parent = file.getParentFile();
        String name = file.getName().substring(0, file.getName().lastIndexOf("."));
        File target = new File(parent, name + ".expanded/");

        // if file is already expanded, i.e. either we expanded it
        // previously, or artifact was installed into local repo
        // using maven eoa plugin - as opposed to dowloading it from
        // a remote repository

        if (target.exists()) {
            return target;
        }

        // if artifact is a snapshot and it's not already expanded,
        // then it was downloaded from a remote repository. we do
        // not expand these, i.e. the image is not static

        if (isSnapshot) {
            return file;
        }

        // it's a 'release' version, i.e. the image is static, we
        // can expand in place and return result

        try {
            unpackArchive(file, target, false);
        } catch (Exception ex) {
            throw new MavenException("Unable to expand deployment artifact. " + ex);
        }

        return target;

    }

    @SuppressWarnings("unchecked")
    private Map<String, Artifact> createManagedVersionMap(MavenProject project) throws InvalidVersionSpecificationException {
        DependencyManagement dependencyManagement = project.getDependencyManagement();
        if (dependencyManagement == null) {
            return Collections.EMPTY_MAP;
        }
        if (dependencyManagement.getDependencies() == null) {
            return Collections.EMPTY_MAP;
        }
        Map<String, Artifact> managedVersionMap = new HashMap<String, Artifact>();
        Iterator<?> itr = dependencyManagement.getDependencies().iterator();
        while (itr.hasNext()) {
            Dependency d = (Dependency)itr.next();
            VersionRange versionRange = VersionRange.createFromVersionSpec(d.getVersion());
            Artifact artifact = artifactFactory.createDependencyArtifact(d.getGroupId(), d.getArtifactId(),
                    versionRange, d.getType(), d.getClassifier(), d.getScope(), d.isOptional());
            managedVersionMap.put(d.getManagementKey(), artifact);
        }
        return managedVersionMap;
    }

    private void unpackArchive(File archive, File outputDir, boolean isDestinationTemporary) throws Exception {

        outputDir.mkdir();
        if (isDestinationTemporary) {
            outputDir.deleteOnExit();
        }

        ZipFile zipFile = new ZipFile(archive);
        try {
            Enumeration<?> entries = zipFile.entries();
            while (entries.hasMoreElements()) {
                ZipEntry entry = (ZipEntry)entries.nextElement();
                File file = new File(outputDir, File.separator + entry.getName());
                if (entry.isDirectory()) {
                    mkdirs(file);
                } else {
                    InputStream in = null;
                    OutputStream out = null;
                    try {
                        in = zipFile.getInputStream(entry);
                        out = new BufferedOutputStream(new FileOutputStream(file));
                        byte[] buffer = new byte[4096];
                        for (int length = 0; length >= 0; length = in.read(buffer)) {
                            out.write(buffer, 0, length);
                        }
                    } finally {
                        if (in != null) {
                            in.close();
                        }
                        if (out != null) {
                            out.close();
                        }
                    }
                    if (isDestinationTemporary) {
                        file.deleteOnExit();
                    }
                }
            }
        } finally {
            if (zipFile != null) {
                zipFile.close();
            }
        }

    }

    private void mkdirs(File child) {
        if (child.exists() == false) {
            child.mkdir();
            child.deleteOnExit();
        }
        File parent = child.getParentFile();
        while (parent != null && parent.exists() == false) {
            parent.mkdir();
            parent.deleteOnExit();
            parent = parent.getParentFile();
        }
    }

    static class OrArtifactFilter implements ArtifactFilter {

        private final List<ArtifactFilter> filters = new ArrayList<ArtifactFilter>();

        public boolean include(Artifact artifact) {
            boolean include = false;
            for (ArtifactFilter filter : filters) {
                if (filter.include(artifact)) {
                    include = true;
                    break;
                }
            }
            return include;
        }

        public void add(ArtifactFilter artifactFilter) {
            filters.add(artifactFilter);
        }

    }

}
